/*
		World.cpp : implementation file

        Copyright (C) 2000 Patrick Dughi

        This software may be used and distributed according to the terms
        of the GNU Public License, incorporated herein by reference, located
		as the file gpl.txt in this same directory.

        The author may be reached at dughi@imaxx.net.
.		
		World class structure and related functions (load/save/etc). Placeholder for room/obj/etc lists.
*/

#pragma warning(disable:4786)
#include "stdafx.h"
#include "Cbreak.h"
#include "World.h"
#include "Room.h"
#include "utils.h"
#include "CircleFile.h"
#include <map>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern CDirDialog ourlib;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

World::World()
{
  more_doors = 0;
	zone_list.clear();
	mob_list.clear();
	obj_list.clear();
	shop_list.clear();
	obj_files.clear();
	wld_files.clear();
	zon_files.clear();
	shp_files.clear();
}

World::~World()
{

}

/* all we need for the world to boot is a directory.  We automatically assume that    */
/* said directory is set up with the 5 dirs, each with at least an index file in them */

void World::boot_world()
{
	Zone_Data *new_zone;
	Room *new_room;
	Mob *new_mob;
	Obj *new_obj;
	Shop *new_shop;

    CircleFile fp,fl; 
	string temp,temp2;      
  map<int,Zone_Data>::iterator zrn;

	/* zones, of course, are first */
  if(ourlib.m_strPath.GetAt(ourlib.m_strPath.GetLength()-1) != '\\') {
  	temp = iprintf("%s\\%s\\index",ourlib.m_strPath,"zon");
  } else {
    temp = iprintf("%s%s\\index",ourlib.m_strPath,"zon");
  }
	fp.open_file(temp,0,0);  

	/* this may confuse people.  Good.  Zones are confusing.                            */
  /*																																									*/
	/*   That's because we have one file which lists all commands for all rooms					*/
	/*   in a situation where it's obvious that all commands are per-room based.				*/	
  /* Further, it's obvious that because order of commands in the file is important,		*/
	/* all sorts of nasty things can happen, and it is difficult to account for all			*/
	/* of them, and requires in-place editing in situations where that's annoying				*/
	/* (ascii files). So, we'll grab those command lines _after_ we've loaded the rooms */


	fp.get_line(&temp);
	while(!fp.eof() && (!temp.empty() &&temp.at(0) != '$') && strstr(temp.c_str(),".")) {
    new_zone = new Zone_Data;
		new_zone->load_zone(atoi(temp.substr(0,temp.find(".")).c_str()));
    zone_list[atoi(temp.substr(0,temp.find(".")).c_str())] = *new_zone;
		zon_files.push_back(atoi(temp.substr(0,temp.find(".")).c_str()));
		fp.get_line(&temp);

  }
	fp.close_file();

	/* now, rooms.  Technically, we should only grab the rooms which we have zones for.   */
	/* it may be slightly time consuming, but it's better than having an error. Each room */
  temp.erase();
  if(ourlib.m_strPath.GetAt(ourlib.m_strPath.GetLength()-1) != '\\') {
  	temp = iprintf("%s\\%s\\index",ourlib.m_strPath,"wld");
  } else {
    temp = iprintf("%s%s\\index",ourlib.m_strPath,"wld");
  }
	fp.open_file(temp,0,0);

	fp.get_line(&temp);
	/* could look for dollar sign completion, but why bother? */
  while(!fp.eof() && (!temp.empty() && temp.at(0) != '$') && strstr(temp.c_str(),".")) {
		temp2 = iprintf("%s\\wld\\%s",ourlib.m_strPath,temp.c_str());
    wld_files.push_back(atoi(temp.substr(0,temp.find(".")).c_str()));
		fl.open_file(temp2,0,atoi(temp.substr(0,temp.find(".")).c_str()));
	
		fl.get_line(&temp);

    while(!fl.eof()) {
  		if(temp.empty() || temp.at(0) != '#') {
				fl.get_line(&temp);
	  		continue;
			}
  		new_room = new Room;
	  	new_room->number = atoi(temp.substr(1).c_str());
			if(new_room->number != 99999 && new_room->load_room(&fl)) {
  	  		  room_list[new_room->number]=*new_room;
			} else {
			  delete new_room;
			}
		  fl.get_line(&temp);	
    }
		fl.close_file();
	  fp.get_line(&temp);
	}
  fp.close_file();

	/*		Now for loading our mobs! */
  if(ourlib.m_strPath.GetAt(ourlib.m_strPath.GetLength()-1) != '\\') {
  	temp = iprintf("%s\\%s\\index",ourlib.m_strPath,"mob");
  } else {
    temp = iprintf("%s%s\\index",ourlib.m_strPath,"mob");
  }
	fp.open_file(temp,0,0);  

	fp.get_line(&temp);
	while(!fp.eof() && (!temp.empty() &&temp.at(0) != '$') && strstr(temp.c_str(),".")) {
		mob_files.push_back(atoi(temp.substr(0,temp.find(".")).c_str()));
		temp2 = iprintf("%s\\mob\\%s",ourlib.m_strPath,temp.c_str());
		fl.open_file(temp2,0,atoi(temp.substr(0,temp.find(".")).c_str()));
	
		fl.get_line(&temp);

    while(!fl.eof()) {
  		if(temp.empty() || temp.at(0) != '#') {
				fl.get_line(&temp);
	  		continue;
			}
  		new_mob = new Mob;
	  	new_mob->number = atoi(temp.substr(1).c_str());
			if(new_mob->number != 99999 && new_mob->load_mob(&fl)) {
  	  		  mob_list[new_mob->number]=*new_mob;
			} else {
			  delete new_mob;
			}
		  fl.get_line(&temp);	
    }
		fl.close_file();
		fp.get_line(&temp);
  }
	fp.close_file();

	/*		Yep, loading our objs */
  if(ourlib.m_strPath.GetAt(ourlib.m_strPath.GetLength()-1) != '\\') {
  	temp = iprintf("%s\\%s\\index",ourlib.m_strPath,"obj");
  } else {
    temp = iprintf("%s%s\\index",ourlib.m_strPath,"obj");
  }
	fp.open_file(temp,0,0);  

	fp.get_line(&temp);
	while(!fp.eof() && (!temp.empty() &&temp.at(0) != '$') && strstr(temp.c_str(),".")) {
		obj_files.push_back(atoi(temp.substr(0,temp.find(".")).c_str()));
		temp2 = iprintf("%s\\obj\\%s",ourlib.m_strPath,temp.c_str());
		fl.open_file(temp2,0,atoi(temp.substr(0,temp.find(".")).c_str()));
	
		fl.get_line(&temp);

    while(!fl.eof()) {
  		if(temp.empty() || temp.at(0) != '#') {
				fl.get_line(&temp);
	  		continue;
			}
  		new_obj = new Obj;
	  	new_obj->number = atoi(temp.substr(1).c_str());
			if(new_obj->number != 99999 && new_obj->load_obj(&fl)) {
  	  		  obj_list[new_obj->number]=*new_obj;
			} else {
			  delete new_obj;
			}
		  fl.get_line(&temp);	
    }
		fl.close_file();
		fp.get_line(&temp);
  }
	fp.close_file();

	/* we even load shops */
  if(ourlib.m_strPath.GetAt(ourlib.m_strPath.GetLength()-1) != '\\') {
  	temp = iprintf("%s\\%s\\index",ourlib.m_strPath,"shp");
  } else {
    temp = iprintf("%s%s\\index",ourlib.m_strPath,"shp");
  }
	fp.open_file(temp,0,0);  

	fp.get_line(&temp);
	while(!fp.eof() && (!temp.empty() &&temp.at(0) != '$') && strstr(temp.c_str(),".")) {
		shp_files.push_back(atoi(temp.substr(0,temp.find(".")).c_str()));
		temp2 = iprintf("%s\\shp\\%s",ourlib.m_strPath,temp.c_str());
    fl.open_file(temp2,0,atoi(temp.substr(0,temp.find(".")).c_str()));
	
		fl.get_line(&temp);  /* shop identifier */

	  fl.get_line(&temp);
    while(!fl.eof()) {
  		if(temp.empty() || temp.at(0) != '#') {
				fl.get_line(&temp);
	  		continue;
			}

  		new_shop = new Shop;
	  	new_shop->number = atoi(temp.substr(1,temp.find("~")).c_str());
			if(new_shop->number != 99999 && new_shop->load_shop(&fl)) {
  	  		  shop_list[new_shop->number]=*new_shop;
			} else {
			  delete new_shop;
			}
		  fl.get_line(&temp);	
    }
		fl.close_file();
		fp.get_line(&temp);
  }
	fp.close_file();

	/* and now, zone commands.. lucky for us, they're easy! */
	for(zrn = zone_list.begin(); zrn != zone_list.end();zrn++) {
		zrn->second.load_zonecmds();
	}
}

void World::save_zone(int zn)
{
	map<int,Room>::iterator ri;
	map<int,Mob>::iterator mi;
	map<int,Obj>::iterator oi;
	map<int,Shop>::iterator si;

  CircleFile cf;
  string temp,temp2;

	temp = iprintf("%s\\wld\\%d.wld",ourlib.m_strPath,zn);
	temp2 = iprintf("%s.new",temp.c_str());
	cf.open_file(temp2,1,zn);

	for(ri=room_list.begin();ri!=room_list.end();ri++) {
		if(ri->second.zone == zn) {
			 ri->second.write_room(&cf);
		}
	}
	cf.write_line("$\n");
	cf.close_file();

	unlink(temp.c_str());
	rename(temp2.c_str(),temp.c_str());

	
	temp = iprintf("%s\\mob\\%d.mob",ourlib.m_strPath,zn);
	temp2 = iprintf("%s.new",temp.c_str());
	cf.open_file(temp2,1,zn);

	for(mi=mob_list.begin();mi!=mob_list.end();mi++) {
		if(mi->second.zone == zn) {
			 mi->second.write_mob(&cf);
		}
	}
	cf.write_line("$\n");
	cf.close_file();

	unlink(temp.c_str());
	rename(temp2.c_str(),temp.c_str());

  temp = iprintf("%s\\obj\\%d.obj",ourlib.m_strPath,zn);
	temp2 = iprintf("%s.new",temp.c_str());
	cf.open_file(temp2,1,zn);

	for(oi=obj_list.begin();oi!=obj_list.end();oi++) {
		if(oi->second.zone == zn) {
			 oi->second.write_obj(&cf);
		}
	}
	cf.write_line("$~\n");
	cf.close_file();

	unlink(temp.c_str());
	rename(temp2.c_str(),temp.c_str());

	temp = iprintf("%s\\shp\\%d.shp",ourlib.m_strPath,zn);
	temp2 = iprintf("%s.new",temp.c_str());
	cf.open_file(temp2,1,zn);
  cf.write_line("CircleMUD v3.0 Shop File~\n");
	for(si=shop_list.begin();si!=shop_list.end();si++) {
		if(si->second.zone == zn) {
			 si->second.write_shop(&cf);
		}
	}
	cf.write_line("$~\n");
	cf.close_file();

	unlink(temp.c_str());
	rename(temp2.c_str(),temp.c_str());

	temp = iprintf("%s\\zon\\%d.zon",ourlib.m_strPath,zn);
	temp2 = iprintf("%s.new",temp.c_str());
	cf.open_file(temp2,1,zn);
    zone_list[zn].write_zone(&cf);
	cf.close_file();
//	unlink(temp.c_str());
//	rename(temp2.c_str(),temp.c_str());
}
